
// Windows 8 Patcher

#include <Windows.h>
#include <Shlobj.h>
#include <stdio.h>


// forward declarations
BOOL IsUserElevatedAdmin();

// external functions
int TerminalServerPatch();
int RemoveWatermarks(wchar_t * MuiPath, DWORD * ListStringIDs, int RebuildMuiCache, int RemoveDesktopBackgroundCache);
void Shutdown();

// list of watermarks (string IDs)
DWORD Watermarks8102_shell32[] = { 33108, 33109, 33118, 0 };
DWORD Watermarks8250_shell32[] = { 33108, 33109, 0 };
DWORD Watermarks8250_basebrd[] = { 12, 0 };
#define Watermarks8400_shell32  Watermarks8250_shell32
#define Watermarks8400_basebrd  Watermarks8250_basebrd


void BtMain()
{
    puts("\nWindows 8 Patcher 1.8 by K. Kleissner\n");

    // Elevated administrator rights?
    if (!IsUserElevatedAdmin())
    {
        puts("Elevated administrator rights are required, exiting.");
        return;
    }

    // no file system redirection if running under WoW64, otherwise this would exchange System32 to SysWOW64
    void * OldValue;
    Wow64DisableWow64FsRedirection(&OldValue);


    // Get the locale
    int LocaleDetected = 0;
    wchar_t Locale[LOCALE_NAME_MAX_LENGTH];
    if (!GetSystemDefaultLocaleName(Locale, LOCALE_NAME_MAX_LENGTH))
        wcscpy(Locale, L"en-US");

    // C:\Windows\System32\en-US\shell32.dll.mui
    // C:\Windows\Branding\Basebrd\en-US\basebrd.dll2.mui
    wchar_t MuiPathShell32_enUS[MAX_PATH], MuiPathBasebrd_enUS[MAX_PATH], MuiPathShell32_local[MAX_PATH], MuiPathBasebrd_local[MAX_PATH];

    wchar_t * System32Dir, * WindowsDir;
    if (SHGetKnownFolderPath(FOLDERID_System, 0, NULL, &System32Dir) != S_OK ||
        SHGetKnownFolderPath(FOLDERID_Windows, 0, NULL, &WindowsDir) != S_OK)
        return;

    wsprintf(MuiPathShell32_enUS, L"%s\\en-US\\shell32.dll.mui", System32Dir);
    wsprintf(MuiPathShell32_local, L"%s\\%s\\shell32.dll.mui", System32Dir, Locale);

    wsprintf(MuiPathBasebrd_enUS, L"%s\\Branding\\Basebrd\\en-US\\basebrd.dll.mui", WindowsDir);
    wsprintf(MuiPathBasebrd_local, L"%s\\Branding\\Basebrd\\%s\\basebrd.dll.mui", WindowsDir, Locale);


    // verify build number
    OSVERSIONINFO VersionInfo;
    VersionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&VersionInfo);

    switch (VersionInfo.dwBuildNumber)
    {
    case 8102:

        // Terminal Server Patch
        puts("Terminal Server Patch termsrv.dll\n------------------------------------------------------------------------");
        TerminalServerPatch();

        // Remove Watermarks
        puts("\n\nRemove Watermarks shell32.dll.mui\n------------------------------------------------------------------------");
        RemoveWatermarks(MuiPathShell32_enUS, Watermarks8102_shell32, 1, 0);

        // also other locale ...?
        if (wcscmp(Locale, L"en-US") != 0 && GetFileAttributes(MuiPathShell32_local) != INVALID_FILE_ATTRIBUTES)
        {
            wprintf(L"\n\nRemove Watermarks shell32.dll.mui %s\n------------------------------------------------------------------------\n", Locale);
            RemoveWatermarks(MuiPathShell32_local, Watermarks8102_shell32, 1, 0);
        }

        break;

    case 8250:
#ifdef _DEBUG
    case 7601:
#endif

        // Terminal Server Patch
        puts("Terminal Server Patch termsrv.dll\n------------------------------------------------------------------------");
        TerminalServerPatch();

        // Remove Watermarks
        if (GetFileAttributes(MuiPathShell32_enUS) != INVALID_FILE_ATTRIBUTES)
        {
            puts("\n\nRemove Watermarks shell32.dll.mui\n------------------------------------------------------------------------");
            RemoveWatermarks(MuiPathShell32_enUS, Watermarks8250_shell32, 0, 0);

            puts("\n\nRemove Watermarks basebrd.dll.mui\n------------------------------------------------------------------------");
            RemoveWatermarks(MuiPathBasebrd_enUS, Watermarks8250_basebrd, 1, 1);
        }

        // also other locale ...?
        if (wcscmp(Locale, L"en-US") != 0 && GetFileAttributes(MuiPathShell32_local) != INVALID_FILE_ATTRIBUTES)
        {
            wprintf(L"\n\nRemove Watermarks shell32.dll.mui %s\n------------------------------------------------------------------------\n", Locale);
            RemoveWatermarks(MuiPathShell32_local, Watermarks8250_shell32, 0, 0);

            wprintf(L"\n\nRemove Watermarks basebrd.dll.mui %s\n------------------------------------------------------------------------\n", Locale);
            RemoveWatermarks(MuiPathBasebrd_local, Watermarks8250_basebrd, 1, 1);
        }

        break;

    case 8400:

        // Terminal Server Patch
        puts("Terminal Server Patch termsrv.dll\n------------------------------------------------------------------------");
        TerminalServerPatch();

        // Remove Watermarks
        if (GetFileAttributes(MuiPathShell32_enUS) != INVALID_FILE_ATTRIBUTES)
        {
            puts("\n\nRemove Watermarks shell32.dll.mui en-US\n------------------------------------------------------------------------");
            RemoveWatermarks(MuiPathShell32_enUS, Watermarks8400_shell32, 0, 0);

            puts("\n\nRemove Watermarks basebrd.dll.mui en-US\n------------------------------------------------------------------------");
            RemoveWatermarks(MuiPathBasebrd_enUS, Watermarks8400_basebrd, 1, 1);

            LocaleDetected = 1;
        }

        // also other locale ...?
        if (wcscmp(Locale, L"en-US") != 0 && GetFileAttributes(MuiPathShell32_local) != INVALID_FILE_ATTRIBUTES)
        {
            wprintf(L"\n\nRemove Watermarks shell32.dll.mui %s\n------------------------------------------------------------------------\n", Locale);
            RemoveWatermarks(MuiPathShell32_local, Watermarks8400_shell32, 0, 0);

            wprintf(L"\n\nRemove Watermarks basebrd.dll.mui %s\n------------------------------------------------------------------------\n", Locale);
            RemoveWatermarks(MuiPathBasebrd_local, Watermarks8400_basebrd, 1, 1);

            LocaleDetected = 1;
        }

        if (!LocaleDetected)
            wprintf(L"\n\nCould not detect locale for removing watermarks! Tried en-US and %s.\nYou will have to compile yourself a hard-coded version.\n", Locale);

        break;

    case 9200:

        // Terminal Server Patch
        puts("Terminal Server Patch termsrv.dll\n------------------------------------------------------------------------");
        TerminalServerPatch();

        break;

    default:
        puts("Error: Unsupported build number. Currently only supported: 8102, 8250, 8400, 9200\n");
        return;
    }

    // Restart System
    puts("\n\nRestarting in 8 seconds..");
    Sleep(8 * 1000);
#ifndef _DEBUG
    Shutdown();
#endif

}



/* This only checks if the user is Administrator and has elevated rights.
   On admin with UAC on it will report 0. AllocateAndInitializeSid and CheckTokenMembership work on guest/admin (with UAC on/off).
*/

BOOL IsUserElevatedAdmin()
{
    SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
    PSID SecurityIdentifier;
    if (!AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &SecurityIdentifier))
        return 0;

    BOOL IsAdminMember;
    if (!CheckTokenMembership(NULL, SecurityIdentifier, &IsAdminMember))
        IsAdminMember = FALSE;

    FreeSid(SecurityIdentifier);

    return IsAdminMember;
}
