
#include <Windows.h>
#include <stdio.h>
#include <accctrl.h>
#include <aclapi.h>
#include <Shlobj.h>
#include <psapi.h>
#include "ntapi.h"

#define printf  



// from http://msdn.microsoft.com/en-us/library/windows/desktop/aa379620(v=vs.85).aspx


//Forward declaration of SetPrivilege
BOOL SetPrivilege(
    HANDLE hToken,          // access token handle
    LPCTSTR lpszPrivilege,  // name of privilege to enable/disable
    BOOL bEnablePrivilege   // to enable or disable privilege
    ) ;


BOOL TakeOwnership(LPTSTR lpszOwnFile) 
{

    BOOL bRetval = FALSE;

    HANDLE hToken = NULL; 
    PSID pSIDAdmin = NULL;
    PSID pSIDEveryone = NULL;
    PACL pACL = NULL;
    SID_IDENTIFIER_AUTHORITY SIDAuthWorld =
            SECURITY_WORLD_SID_AUTHORITY;
    SID_IDENTIFIER_AUTHORITY SIDAuthNT = SECURITY_NT_AUTHORITY;
    const int NUM_ACES  = 2;
    EXPLICIT_ACCESS ea[NUM_ACES];
    DWORD dwRes;

    // Specify the DACL to use.
    // Create a SID for the Everyone group.
    if (!AllocateAndInitializeSid(&SIDAuthWorld, 1,
                     SECURITY_WORLD_RID,
                     0,
                     0, 0, 0, 0, 0, 0,
                     &pSIDEveryone)) 
    {
        printf("AllocateAndInitializeSid (Everyone) error %u\n",
                GetLastError());
        goto Cleanup;
    }

    // Create a SID for the BUILTIN\Administrators group.
    if (!AllocateAndInitializeSid(&SIDAuthNT, 2,
                     SECURITY_BUILTIN_DOMAIN_RID,
                     DOMAIN_ALIAS_RID_ADMINS,
                     0, 0, 0, 0, 0, 0,
                     &pSIDAdmin)) 
    {
        printf("AllocateAndInitializeSid (Admin) error %u\n",
                GetLastError());
        goto Cleanup;
    }

    ZeroMemory(&ea, NUM_ACES * sizeof(EXPLICIT_ACCESS));

    // Set read access for Everyone.
    ea[0].grfAccessPermissions = /*GENERIC_READ*/GENERIC_ALL;
    ea[0].grfAccessMode = SET_ACCESS;
    ea[0].grfInheritance = NO_INHERITANCE;
    ea[0].Trustee.TrusteeForm = TRUSTEE_IS_SID;
    ea[0].Trustee.TrusteeType = TRUSTEE_IS_WELL_KNOWN_GROUP;
    ea[0].Trustee.ptstrName = (LPTSTR) pSIDEveryone;

    // Set full control for Administrators.
    ea[1].grfAccessPermissions = GENERIC_ALL;
    ea[1].grfAccessMode = SET_ACCESS;
    ea[1].grfInheritance = NO_INHERITANCE;
    ea[1].Trustee.TrusteeForm = TRUSTEE_IS_SID;
    ea[1].Trustee.TrusteeType = TRUSTEE_IS_GROUP;
    ea[1].Trustee.ptstrName = (LPTSTR) pSIDAdmin;

    if (ERROR_SUCCESS != SetEntriesInAcl(NUM_ACES,
                                         ea,
                                         NULL,
                                         &pACL))
    {
        printf("Failed SetEntriesInAcl\n");
        goto Cleanup;
    }

    // Try to modify the object's DACL.
    dwRes = SetNamedSecurityInfo(
        lpszOwnFile,                 // name of the object
        SE_FILE_OBJECT,              // type of object
        DACL_SECURITY_INFORMATION,   // change only the object's DACL
        NULL, NULL,                  // do not change owner or group
        pACL,                        // DACL specified
        NULL);                       // do not change SACL

    if (ERROR_SUCCESS == dwRes) 
    {
        printf("Successfully changed DACL\n");
        bRetval = TRUE;
        // No more processing needed.
        goto Cleanup;
    }
    if (dwRes != ERROR_ACCESS_DENIED)
    {
        printf("First SetNamedSecurityInfo call failed: %u\n",
                dwRes); 
        goto Cleanup;
    }

    // If the preceding call failed because access was denied, 
    // enable the SE_TAKE_OWNERSHIP_NAME privilege, create a SID for 
    // the Administrators group, take ownership of the object, and 
    // disable the privilege. Then try again to set the object's DACL.

    // Open a handle to the access token for the calling process.
    if (!OpenProcessToken(GetCurrentProcess(), 
                          TOKEN_ADJUST_PRIVILEGES, 
                          &hToken)) 
       {
          printf("OpenProcessToken failed: %u\n", GetLastError()); 
          goto Cleanup; 
       } 

    // Enable the SE_TAKE_OWNERSHIP_NAME privilege.
    if (!SetPrivilege(hToken, SE_TAKE_OWNERSHIP_NAME, TRUE)) 
    {
        printf("You must be logged on as Administrator.\n");
        goto Cleanup; 
    }

    // Set the owner in the object's security descriptor.
    dwRes = SetNamedSecurityInfo(
        lpszOwnFile,                 // name of the object
        SE_FILE_OBJECT,              // type of object
        OWNER_SECURITY_INFORMATION,  // change only the object's owner
        pSIDAdmin,                   // SID of Administrator group
        NULL,
        NULL,
        NULL); 

    if (dwRes != ERROR_SUCCESS) 
    {
        printf("Could not set owner. Error: %u\n", dwRes); 
        goto Cleanup;
    }
        
    // Disable the SE_TAKE_OWNERSHIP_NAME privilege.
    if (!SetPrivilege(hToken, SE_TAKE_OWNERSHIP_NAME, FALSE)) 
    {
        printf("Failed SetPrivilege call unexpectedly.\n");
        goto Cleanup;
    }

    // Try again to modify the object's DACL,
    // now that we are the owner.
    dwRes = SetNamedSecurityInfo(
        lpszOwnFile,                 // name of the object
        SE_FILE_OBJECT,              // type of object
        DACL_SECURITY_INFORMATION,   // change only the object's DACL
        NULL, NULL,                  // do not change owner or group
        pACL,                        // DACL specified
        NULL);                       // do not change SACL

    if (dwRes == ERROR_SUCCESS)
    {
        printf("Successfully changed DACL\n");
        bRetval = TRUE; 
    }
    else
    {
        printf("Second SetNamedSecurityInfo call failed: %u\n",
                dwRes); 
    }

Cleanup:

    if (pSIDAdmin)
        FreeSid(pSIDAdmin); 

    if (pSIDEveryone)
        FreeSid(pSIDEveryone); 

    if (pACL)
       LocalFree(pACL);

    if (hToken)
       CloseHandle(hToken);

    return bRetval;

}



// from http://msdn.microsoft.com/en-us/library/windows/desktop/aa446619(v=vs.85).aspx, requires cmcfg32.lib


BOOL SetPrivilege(
    HANDLE hToken,          // access token handle
    LPCTSTR lpszPrivilege,  // name of privilege to enable/disable
    BOOL bEnablePrivilege   // to enable or disable privilege
    ) 
{
    TOKEN_PRIVILEGES tp;
    LUID luid;

    if ( !LookupPrivilegeValue( 
            NULL,            // lookup privilege on local system
            lpszPrivilege,   // privilege to lookup 
            &luid ) )        // receives LUID of privilege
    {
        printf("LookupPrivilegeValue error: %u\n", GetLastError() ); 
        return FALSE; 
    }

    tp.PrivilegeCount = 1;
    tp.Privileges[0].Luid = luid;
    if (bEnablePrivilege)
        tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    else
        tp.Privileges[0].Attributes = 0;

    // Enable the privilege or disable all privileges.

    if ( !AdjustTokenPrivileges(
           hToken, 
           FALSE, 
           &tp, 
           sizeof(TOKEN_PRIVILEGES), 
           (PTOKEN_PRIVILEGES) NULL, 
           (PDWORD) NULL) )
    { 
          printf("AdjustTokenPrivileges error: %u\n", GetLastError() ); 
          return FALSE; 
    } 

    if (GetLastError() == ERROR_NOT_ALL_ASSIGNED)

    {
          printf("The token does not have the specified privilege. \n");
          return FALSE;
    } 

    return TRUE;
}



void Shutdown()
{
    HANDLE Token;
    TOKEN_PRIVILEGES tkp;

    if (!OpenProcessToken(GetCurrentProcess(),  TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &Token))
        return;

    LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);
    tkp.PrivilegeCount = 1;  // one privilege to set     
    tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;  
  
    AdjustTokenPrivileges(Token, FALSE, &tkp, 0,  (PTOKEN_PRIVILEGES)NULL, 0);  
  
    NtShutdownSystem(/*ShutdownPowerOff*/ShutdownReboot);
} 



// not used anymore, redirection can be disabled through Wow64DisableWow64FsRedirection

int GetSystem32Directory(wchar_t * System32Directory)
{
    // beware of File System Redirector on Wow64 (http://msdn.microsoft.com/en-us/library/aa384187(v=vs.85).aspx)
    int Is64 = 0;
    IsWow64Process(GetCurrentProcess(), &Is64);

    if (Is64)
    {
        wchar_t * WindowsDir;
        if (SHGetKnownFolderPath(FOLDERID_Windows, 0, NULL, &WindowsDir) != S_OK)
            return 0;

        wcscpy(System32Directory, WindowsDir);
        wcscat(System32Directory, L"\\Sysnative");
    }
    else
    {
        wchar_t * System32Dir;
        if (SHGetKnownFolderPath(FOLDERID_System, 0, NULL, &System32Dir) != S_OK)
            return 0;

        wcscpy(System32Directory, System32Dir);
    }

    return 1;
}



int CreateHiddenProcess(wchar_t * ApplicationName, wchar_t * CommandLine, unsigned Time)
{
    STARTUPINFOW StartupInfo;
    PROCESS_INFORMATION ProcessInfo;
    wchar_t CommandLineM[MAX_PATH];

    memset(&StartupInfo, 0, sizeof(StartupInfo));
    StartupInfo.cb = sizeof(StartupInfo);
    StartupInfo.dwFlags = STARTF_USESHOWWINDOW;
    StartupInfo.wShowWindow = SW_HIDE;

    wcscpy(CommandLineM, CommandLine);

    if (!CreateProcessW(ApplicationName, CommandLineM, NULL, NULL, false, 0, NULL, NULL, &StartupInfo, &ProcessInfo))
        return 0;

    CloseHandle(ProcessInfo.hThread);

    // wait for the proccess to exit
    if (Time && WaitForSingleObject(ProcessInfo.hProcess, Time) != WAIT_OBJECT_0)
    {
        CloseHandle(ProcessInfo.hProcess);
        return 0;
    }

    CloseHandle(ProcessInfo.hProcess);

    return 1;
}



/*  Closes the service (waits for it)
        0 = Error opening services
        1 = Error opening RDP service
        2 = Error closing RDP service
        3 = Successful
*/

int StopServiceWait(wchar_t * ServiceName, unsigned Time)
{
    SC_HANDLE SCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
    if (!SCManager)
        return 0;

    SC_HANDLE ServHand = OpenService(SCManager, ServiceName, SERVICE_ALL_ACCESS);
    if (!ServHand)
    {
        int LastError = GetLastError();
        CloseServiceHandle(SCManager);
        return 1;
    }

    SERVICE_STATUS Status;
    int ReturnStatus = 3;

    if (!ControlService(ServHand, SERVICE_CONTROL_STOP, &Status))
    {
        // if the service is already stopped we are good to go
        if (GetLastError() != ERROR_SERVICE_NOT_ACTIVE)
            ReturnStatus = 2;

        goto ExitServiceWait;
    }

    // wait for the service to be closed
    while (Time)
    {
        if (!QueryServiceStatus(ServHand, &Status))
        {
            ReturnStatus = 2;
            goto ExitServiceWait;
        }

        if (Status.dwCurrentState == SERVICE_STOPPED)
            break;

        Time -= 200;
        Sleep(200);
    }

    if (!Time)  // time out? :(
        ReturnStatus = 2;

    ExitServiceWait:

    CloseServiceHandle(SCManager);
    CloseServiceHandle(ServHand);

    return ReturnStatus;
}



/*  Terminates processes with the image name
        Return = count of processes closed
*/

int TerminateProcessWait(wchar_t * ImageName, unsigned Time)
{
    int Status = 0;
    DWORD ProcessIDs[500];
    DWORD BytesReturn;

    if (!EnumProcesses(ProcessIDs, sizeof(ProcessIDs), &BytesReturn))
        return 0;

    for (unsigned n = 0; n < BytesReturn /4; n++)
    {
        HANDLE Process = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_TERMINATE | SYNCHRONIZE, false, ProcessIDs[n]);
        if (!Process)
            continue;

        wchar_t ExecutablePath[MAX_PATH];
        if (!GetProcessImageFileName(Process, ExecutablePath, MAX_PATH))
        {
            CloseHandle(Process);
            continue;
        }

        wchar_t * ImageNameRemote = wcsrchr(ExecutablePath, L'\\') + 1;

        if (!ImageNameRemote)
        {
            CloseHandle(Process);
            continue;
        }

        if (_wcsicmp(ImageNameRemote, ImageName) == 0)
        {
            // found!
            if (TerminateProcess(Process, 1))                   // setting exit code to 1 makes it not restarting immediately!
            {
                Status++;
                WaitForSingleObject(Process, Time);
            }
        }
    }

    return Status;
}


/*  Custom MoveFileExW because Windows one exchanges System32 with SysWOW64 when calling from 32-bit process
        0 = Error opening or setting key
        1 = Successful
*/

int MoveFileExWCustom(wchar_t * lpExistingFileName, wchar_t * lpNewFileName, DWORD Flags)
{
	HKEY Key;

    if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Control\\Session Manager", 0, KEY_SET_VALUE, &Key) != ERROR_SUCCESS)
        return 0;

    wchar_t Data[MAX_PATH * 2];
    wcscpy(Data, L"\\??\\");
    wcscat(Data, lpExistingFileName);

    int Length = wcslen(Data) + 1;

    wcscpy(Data + Length, L"!\\??\\");
    wcscpy(Data + Length + 5, lpNewFileName);

    Length += wcslen(Data + Length) + 1;
    Data[Length++] = 0;

    if (RegSetValueEx(Key, L"PendingFileRenameOperations", 0, REG_MULTI_SZ, (BYTE *)&Data, Length * 2) != ERROR_SUCCESS)
        return 0;

    RegCloseKey(Key);

    return 1;
}



int SetRunOnce(wchar_t * Name, wchar_t * Command)
{
	HKEY Key;

    if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"Software\\Microsoft\\Windows\\CurrentVersion\\RunOnce", 0, KEY_SET_VALUE, &Key) != ERROR_SUCCESS ||
        RegSetValueEx(Key, Name, 0, REG_SZ, (BYTE *)Command, wcslen(Command) * 2 + 2) != ERROR_SUCCESS)
        return 0;

    RegCloseKey(Key);

    return 1;
}


