
// Terminal Server Patch for 8

#include <Windows.h>
#include <Shlobj.h>
#include <stdio.h>

#include "Support.h"

int ReadFileMemory(wchar_t * FileName, void ** Memory, unsigned * Size);
int VerifyPatch(void * Memory, unsigned Size);
int ApplyPatch(void * Memory);
int FixRegistryValues();



int TerminalServerPatch()
{
    // C:\Windows\System32\termsrv.dll
    wchar_t TermsrvPath[MAX_PATH];

    wchar_t * System32Dir;
    if (SHGetKnownFolderPath(FOLDERID_System, 0, NULL, &System32Dir) != S_OK)
        return 0;
    wcscpy(TermsrvPath, System32Dir);

#ifdef _DEBUG
    wcscat(TermsrvPath, L"\\termsrv2.dll");
#else
    wcscat(TermsrvPath, L"\\termsrv.dll");
#endif


    // Stop RDP service (otherwise the dll is in use and cannot be opened for write access on 64-bit)
    printf("%s", "Stop RDP service\t\t\t");

    StopServiceWait(L"Mcx2Svc", 4000);                  // that service depends on TermService with Media Center (8 RP), has to be closed first
    StopServiceWait(L"UmRdpService", 4000);             // that service depends on TermService, has to be closed first

    switch(StopServiceWait(L"TermService", 3000))
    {
    case 0:
        puts("Error opening services");
        break;
    case 1:
        puts("Error opening RDP service");
        break;
    case 2:
        puts("Error closing RDP service2");
        break;
    case 3:
        puts("OK");
        break;
    }


    // Take ownership
    printf("%s", "Take ownership\t\t\t\t");
    if (TakeOwnership(TermsrvPath))
        puts("OK");
    else
    {
        puts("Error!");
        return 0;
    }


    // Already patched?
    printf("%s", "Verify\t\t\t\t\t");

    void * TermsrvBuffer;
    unsigned TermsrvSize;

    switch(ReadFileMemory(TermsrvPath, &TermsrvBuffer, &TermsrvSize))
    {
    case 0:
        puts("Error opening the file!");
        return 0;
    case 1:
        puts("Error getting file size!");
        return 0;
    case 2:
        puts("Error allocating memory!");
        return 0;
    case 3:
        puts("Error reading file!");
        return 0;
    }

    switch(VerifyPatch(TermsrvBuffer, TermsrvSize))
    {
    case 0:
        puts("OK");
        break;
    case 1:
        puts("Already patched!");
        return 0;
    case 2:
        puts("Unknown version!");
        return 0;
    }


    // Patch the file
    printf("%s", "Apply patch\t\t\t\t");

    HANDLE FileHandle = CreateFile(TermsrvPath, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
    if (FileHandle == INVALID_HANDLE_VALUE)
    {
        puts("Error opening for write access!");
        return 0;
    }

    HANDLE Mapping = CreateFileMapping(FileHandle, NULL, PAGE_READWRITE, 0, 0, NULL);
    if (!Mapping)
    {
        puts("Error mapping!");
        return 0;
    }
    BYTE * FileBuffer = (BYTE *)MapViewOfFile(Mapping, FILE_MAP_WRITE, 0, 0, 0);
    if (!FileBuffer)
    {
        puts("Error mapping 2!");
        return 0;
    }

    if (ApplyPatch(FileBuffer))
        puts("OK");
    else
    {
        puts("Error!");
        return 0;
    }

    UnmapViewOfFile(FileBuffer);
    CloseHandle(Mapping);
    CloseHandle(FileHandle);


    // Registry changes
    printf("%s", "Set registry keys\t\t\t");

    switch (FixRegistryValues())
    {
    case 0:
        puts("Error opening key!");
        return 0;
    case 1:
        puts("Error setting key!");
        return 0;
    case 2:
        puts("OK");
    }


    return 1;
}



// memmem not included in ANSI-C, include it here

BYTE * memmem(BYTE * Buffer, BYTE * SearchFor, DWORD BufferSize, DWORD SearchForSize)
{
	BYTE *p;

	for (p = Buffer; p <= (Buffer - SearchForSize + BufferSize); p++)
	{
		if (memcmp(p, SearchFor, SearchForSize) == 0)
			return p; /* found */
	}
	return NULL;
}



/*  Verifies the patch
        0 = Unpatched
        1 = Patched
        2 = Signature not found
*/

DWORD ApplyPatchPosition = 0;
unsigned ApplyPatchSize = 0;
BYTE * ApplyPatchSig = NULL;

BYTE Signature_8102_8250_8400_32_Patched[]    = { 0xB8, 0x00, 0x01, 0x00, 0x00, 0x90, 0x89, 0x86, 0x20, 0x03, 0x00, 0x00 };
BYTE Signature_8102_8250_8400_32_Unpatched[]  = { 0x3B, 0x86, 0x20, 0x03, 0x00, 0x00, 0x0F, 0x84 };

BYTE Signature_8102_8250_8400_64_Patched[]    = { 0xB8, 0x00, 0x01, 0x00, 0x00, 0x89, 0x87, 0x38, 0x06, 0x00, 0x00, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90 };
BYTE Signature_8102_8250_8400_64_Unpatched[]  = { 0x8B, 0x87, 0x38, 0x06, 0x00, 0x00, 0x39, 0x87, 0x3C, 0x06, 0x00, 0x00, 0x0F, 0x84 };

int VerifyPatch(void * Memory, unsigned Size)
{
    // check if the patch was applied already
    void * SignaturePatched;

    if ((SignaturePatched = memmem((BYTE *)Memory, Signature_8102_8250_8400_32_Patched, Size, sizeof(Signature_8102_8250_8400_32_Patched))) ||
        (SignaturePatched = memmem((BYTE *)Memory, Signature_8102_8250_8400_64_Patched, Size, sizeof(Signature_8102_8250_8400_64_Patched))) )
        return 1;

    // look for the signature to patch
    void * SignatureUnpatched;

    if ((SignatureUnpatched = memmem((BYTE *)Memory, Signature_8102_8250_8400_32_Unpatched, Size, sizeof(Signature_8102_8250_8400_32_Unpatched))))
    {
        ApplyPatchSig = Signature_8102_8250_8400_32_Patched;
        ApplyPatchSize = sizeof(Signature_8102_8250_8400_32_Patched);
        ApplyPatchPosition = (DWORD)SignatureUnpatched - (DWORD)Memory;

        return 0;
    }

    else if ((SignatureUnpatched = memmem((BYTE *)Memory, Signature_8102_8250_8400_64_Unpatched, Size, sizeof(Signature_8102_8250_8400_64_Unpatched))))
    {
        ApplyPatchSig = Signature_8102_8250_8400_64_Patched;
        ApplyPatchSize = sizeof(Signature_8102_8250_8400_64_Patched);
        ApplyPatchPosition = (DWORD)SignatureUnpatched - (DWORD)Memory;
        return 0;
    }

    return 2;
}


// Applies the patch

int ApplyPatch(void * Memory)
{
    if (!ApplyPatchPosition)
        return 0;

    memcpy((BYTE *)Memory + ApplyPatchPosition, ApplyPatchSig, ApplyPatchSize);

    return 1;
}


/*  Applies the registry changes
        0 = Error opening key
        1 = Error setting key
        2 = Successful

    Windows Registry Editor Version 5.00

    [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Terminal Server]
    "fDenyTSConnections"=dword:00000000
    "fSingleSessionPerUser"=dword:00000000
*/

int FixRegistryValues()
{
	HKEY TerminalServerKey/*, LsaKey*/;

    if (RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Control\\Terminal Server", 0, KEY_SET_VALUE, &TerminalServerKey) != ERROR_SUCCESS/* ||
        RegOpenKeyExW(HKEY_LOCAL_MACHINE, L"SYSTEM\\CurrentControlSet\\Control\\Lsa", 0, KEY_SET_VALUE, &LsaKey) != ERROR_SUCCESS*/)
        return 0;

    BYTE Data[4] = { 0, 0, 0, 0};
    if (RegSetValueEx(TerminalServerKey, L"fDenyTSConnections", 0, REG_DWORD, Data, 4) != ERROR_SUCCESS ||      // allow TS connections
        RegSetValueEx(TerminalServerKey, L"fSingleSessionPerUser", 0, REG_DWORD, Data, 4) != ERROR_SUCCESS /*||   // allow multiple sessions per user [no effect?]
        RegSetValueEx(LsaKey, L"LimitBlankPasswordUse", 0, REG_DWORD, Data, 4) != ERROR_SUCCESS*/)                // allow blank passwords [deactivated, no effect]
        return 1;

    RegCloseKey(TerminalServerKey);

    return 2;
}



/*  Reads a file temporarily into memory
        0 = Error opening the file
        1 = Error getting file size
        2 = Error allocating memory
        3 = Error reading file
        4 = Successful
*/

int ReadFileMemory(wchar_t * FileName, void ** Memory, unsigned * Size)
{
    HANDLE FileHandle = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
    if (FileHandle == INVALID_HANDLE_VALUE)
        return 0;

    *Size = GetFileSize(FileHandle, NULL);
    if (*Size == INVALID_FILE_SIZE)
    {
        CloseHandle(FileHandle);
        return 1;
    }

    *Memory = VirtualAlloc(NULL, *Size, MEM_COMMIT, PAGE_READWRITE);
    if (!Memory)
    {
        CloseHandle(FileHandle);
        return 2;
    }

    DWORD NumberOfBytesRead;
    if (!ReadFile(FileHandle, *Memory, *Size, &NumberOfBytesRead, NULL))
    {
        CloseHandle(FileHandle);
        return 3;
    }

    CloseHandle(FileHandle);
    return 4;
}

