
// Removing Watermarks for 8

#include <Windows.h>
#include <Shlobj.h>
#include <stdio.h>

#include "Support.h"

int ParseLanguageFile(wchar_t * MuiPath, DWORD * List);

BOOL ActOnReboot = 0;



int RemoveWatermarks(wchar_t * MuiPath, DWORD * ListStringIDs, int RebuildMuiCache, int RemoveDesktopBackgroundCache)
{
    // Take ownership
    printf("%s", "Take ownership\t\t\t\t");
    if (TakeOwnership(MuiPath))
        puts("OK");
    else
    {
        puts("Error!");
        return 0;
    }


    // parse it
    printf("%s", "Verify\t\t\t\t\t");
    if (!ParseLanguageFile(MuiPath, ListStringIDs))
    {
        puts("Error loading file");
        return 0;
    }
    else
        puts("OK");


    // maps the file into memory
    printf("%s", "Patching\t\t\t\t");

    HANDLE FileHandle = CreateFile(MuiPath, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
    if (FileHandle == INVALID_HANDLE_VALUE)
    {
        /* Fallback option, on 64-bit some process has sometimes an open handle to it, explorer.exe, taskhost.exe, TM.exe and SearchProtocolHost.exe were spotted
           Use MoveFileEx (and a copy) instead, trying to close the processes does not seem to (always) help
        */
        wchar_t MuiPathFake[MAX_PATH];
        wcscpy(MuiPathFake, MuiPath);
        wcscat(MuiPathFake, L"2");

        if (!CopyFile(MuiPath, MuiPathFake, 0) ||
            (FileHandle = CreateFile(MuiPathFake, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE ||
            !MoveFileExW(MuiPathFake, MuiPath, MOVEFILE_DELAY_UNTIL_REBOOT | MOVEFILE_REPLACE_EXISTING) )
        {
            puts("Error opening for write access! (please try after reboot again)");
            return 0;
        }

        ActOnReboot = 1;
    }

    HANDLE Mapping = CreateFileMapping(FileHandle, NULL, PAGE_READWRITE, 0, 0, NULL);
    if (!Mapping)
    {
        puts("Error mapping!");
        return 0;
    }
    BYTE * FileBuffer = (BYTE *)MapViewOfFile(Mapping, FILE_MAP_WRITE, 0, 0, 0);
    if (!FileBuffer)
    {
        puts("Error mapping 2!");
        return 0;
    }


    // patch
    for (unsigned n = 0; ListStringIDs[n]; n++)
    {
        if (ListStringIDs[n] == -1)
            continue;

        *(wchar_t *)(FileBuffer + ListStringIDs[n]) = 0;
    }

    puts("OK");



    UnmapViewOfFile(FileBuffer);
    CloseHandle(Mapping);
    CloseHandle(FileHandle);



    // Remove desktop background picture cache?
    if (RemoveDesktopBackgroundCache)
    {
        printf("%s", "Delete theme cache\t\t\t");

        wchar_t WindowsThemes[MAX_PATH];
        wchar_t * AppDataRoaming;
        if (SHGetKnownFolderPath(FOLDERID_RoamingAppData, 0, NULL, &AppDataRoaming) != S_OK)
            return 0;
        wcscpy(WindowsThemes, AppDataRoaming);
        wcscat(WindowsThemes, L"\\Microsoft\\Windows\\Themes\\CachedFiles\\*");

/*        WIN32_FIND_DATA FindData;
        HANDLE Find = FindFirstFile(WindowsThemes, &FindData);
        if (Find == INVALID_HANDLE_VALUE)
            return 0;

        // first: only handle files
        do
        {
            if (FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
                continue;

            wchar_t CompleteFileName[MAX_PATH];
            wcscpy(CompleteFileName, AppDataRoaming);
            wcscat(CompleteFileName, L"\\Microsoft\\Windows\\Themes\\CachedFiles\\");
            wcscat(CompleteFileName, FindData.cFileName);

            DeleteFile(CompleteFileName);
        } while (FindNextFile(Find, &FindData));*/

        // it would be immediately re-created!
        wchar_t DeleteKey[MAX_PATH] = L"cmd /C del /F /Q \"";
        wcscat(DeleteKey, WindowsThemes);
        wcscat(DeleteKey, L"\"");

        SetRunOnce(L"3", DeleteKey);

        puts("OK");
    }


    // Rebuild mui (resource) cache? Running mcbuilder.exe 'Resource cache builder tool'
    if (RebuildMuiCache)
    {
        if (ActOnReboot)
        {
            puts("Rebuild cache\t\t\t\tOn reboot!");

            // set the RunOnce key
            SetRunOnce(L"1", L"mcbuilder.exe");
            SetRunOnce(L"2", L"shutdown -r -t 30");

            printf("%s", "Your system will restart twice\t\tI understand!");
        }
        else
        {
            printf("%s", "Rebuild cache (wait!)\t\t\t");
            CreateHiddenProcess(NULL, L"mcbuilder.exe", INFINITE);
            puts("OK");
        }
    }


    return 1;
}



/*  Parses a mui file and returns pointers to the list
        0 = Error loading file
        1 = Successful
*/

//#define _EXT_OUTPUT

int ParseLanguageFile(wchar_t * MuiPath, DWORD * List)
{
    HMODULE MuiHandle = LoadLibraryEx(MuiPath, NULL, DONT_RESOLVE_DLL_REFERENCES | LOAD_LIBRARY_AS_DATAFILE);
    if (!MuiHandle)
        return 0;

#ifdef _EXT_OUTPUT
	printf("\n\n   ID  Offset  Len  String                                              ");
	printf(  "\n-----  ------  ---  ----------------------------------------------------\n");
#endif

	for (int n = 0; List[n]; n++)
	{
        unsigned StringID = List[n];

        List[n] = -1;                                       // default value

        // find it
        HRSRC Resource = FindResource(MuiHandle, MAKEINTRESOURCE(StringID / 16 + 1), RT_STRING);
        if (!Resource)
        {
#ifdef _EXT_OUTPUT
            printf("%5d               (not found)\n", StringID);
#endif
            continue;
        }

        HGLOBAL Resource1 = LoadResource(MuiHandle, Resource);
        if (!Resource1)
        {
#ifdef _EXT_OUTPUT
            printf("%5d               (error loading)\n", StringID);
#endif
            continue;
        }

        void * ResourceMemory = LockResource(Resource1);
        if (!ResourceMemory)
        {
#ifdef _EXT_OUTPUT
            printf("%5d               (error getting pointer)\n", StringID);
#endif
            continue;
        }

        WCHAR*		pParse		= (WCHAR *)ResourceMemory;		// Pointer to the String block
        UINT		nIndex		= StringID % 16;		        // Calculate the string index

		// 16 strings per block
		for (unsigned i = 0; i < (nIndex & 15); i++)
		{
			pParse += 1 + (int)*pParse;
		}

		// OK, we get it
		int nLen = (UINT)*pParse;                               // The length of the target string.
		pParse += 1;				                            // Pointer to the target string


		// Main point, calculate the string offset
		List[n]	= (DWORD) ( (DWORD_PTR)pParse - (DWORD_PTR)MuiHandle ) + 1;

        wchar_t Text[53];
        if (nLen < 53)
        {
            wcsncpy(Text, pParse, nLen);
            Text[nLen] = 0;
        }
        else
        {
            wcsncpy(Text, pParse, 52 - 3);
            wcscpy(Text + 52 - 3, L"...");
        }

#ifdef _EXT_OUTPUT
        printf("%5d  0x%4X  %3d  %ls\n", StringID, List[n], nLen * sizeof(WCHAR), Text);
#endif
	}

#ifdef _EXT_OUTPUT
    puts("");
#endif

    return 1;
}




